#!/bin/sh
# the next line restarts using tclsh\
exec tclsh "$0" "

#########################################################################
# cmmucl.tcl - Console version of mmucl.
#
# Copyright (C) 1997 Mark Patton
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#########################################################################

# Edit the below paths or let make do it.
# Do NOT edit anything else unless you know tcl/tk.
# Do NOT delete any of the preceding lines. Make depends on them.
set Mmucl(lib_dir) ..
set Mmucl(rc_dir) ..
############################$@"

# Make sure the paths are absolute
if {[file pathtype Mmucl(lib_dir)]=="relative"} {
    set Mmucl(lib_dir) [file join [pwd] $Mmucl(lib_dir)]
}
if {[file pathtype $Mmucl(rc_dir)]=="relative"} {
    set Mmucl(rc_dir) [file join [pwd] $Mmucl(rc_dir)]
}

# Load some init files and make Mmucl(rc_dir) if it doesn't exist
if {![file exists $Mmucl(rc_dir)]} {
    file mkdir $Mmucl(rc_dir)
}
if {[file exists [file join $Mmucl(rc_dir) mmucl.rc]]} {
    source [file join $Mmucl(rc_dir) mmucl.rc]
    } 
if {[file exists [file join $Mmucl(rc_dir) charlist]]} {
    source [file join $Mmucl(rc_dir) charlist]
}

# Make the package mechanism look in the right place
set auto_path [linsert $auto_path 0 $Mmucl(lib_dir)]

package require mmucl

proc display {str} {
    global term_display_row

    term_save_pos
    term_goto_rowcol $term_display_row 0
    puts -nonewline $str
    term_restore_pos
}

proc echo {str {ansi_code ""}} {
    if {$ansi_code!=""} {set str "\033\[${ansi_code}m${str}\033\[0\;37\;40m"}
    display $str
}

proc term_save_pos {} {
    puts -nonewline "\0337"
    flush stdout
}

proc term_restore_pos {} {
    puts -nonewline "\0338"
    flush stdout
}

proc term_goto_rowcol {row col} {
    puts "\033\[${row}\;${col}f"
    flush stdout
}

proc term_erase_screen {} {
    puts -nonewline "\033\[2J"
    flush stdout
}

proc term_erase_toeol {} {
    puts -nonewline "\033\[K"
    flush stdout
}

proc term_reset {} {
    puts -nonewline "\033c"
    flush stdout
}

proc term_scroll_region {top bottom} {
    puts "\033\[${top}\;${bottom}r"
    flush stdout
}

# Make split screen
proc term_initsplit {} {
    global term_display_row

    set rows [lindex [exec stty size] 0] ;# <---- Not portable.

    term_reset
    term_erase_screen
    term_scroll_region 0 [expr {$rows-3}]
    term_goto_rowcol [expr {$rows-2}] 0
    puts " -----"
    set term_display_row [expr {$rows-3}]
}

rename exit _exit
proc exit {} {
    term_reset
    _exit
}

# Load in the readline command
#load [file join $Mmucl(lib_dir) lib tcl_readline.so]

# Figure out how to use lynx for the help system.
#term_reset; exec lynx ../html/echo.html; term_initsplit
#set Mmucl(browser) {lynx %1}

set Mmucl(echo) 1
set Mmucl(echo_color) 33

term_initsplit

# Accept input from the console.
fileevent stdin readable {
    gets stdin Mmucl(input)
    if $Mmucl(echo) {echo $Mmucl(input) $Mmucl(echo_color)}
    mmucl_parse $Mmucl(input)
    term_erase_toeol
}
fconfigure stdin -blocking 0 -buffering line

echo "Preliminary console version of Mmucl.\n"

# Enter the event loop.
vwait forever


# If I ever get readline to enter the Tcl event loop...
# while 1 {
#     set Mmucl(input) [readline]
#     if {$Mmucl(echo)} {echo $Mmucl(input) $Mmucl(echo_color)}
#     mmucl_parse $Mmucl(input)
#     term_erase_toeol
# }
